#include "CompilerThread.h"

CompilerThread::CompilerThread( const UbuDSL_Daemon &, void (*fWriteLog)(const QString &), const QString &driverName, QObject *parent )
: QThread(parent),
  writeLog(fWriteLog),
  m_driverName( driverName ),
  m_result( Success )
{
    writeLog( "ConnectThread::CompilerThread(): called." );
    initDriversMap();
    writeLog( "ConnectThread::CompilerThread(): finished." );
}

CompilerThread::~CompilerThread()
{
    writeLog( "ConnectThread::~CompilerThread(): called and finished." );
}

void CompilerThread::setDriverName( const QString &driverName )
{
    m_driverName = driverName;
}

int CompilerThread::testDriver()
{
    writeLog( "ConnectThread::testDriver(): called." );
    if ( m_driverName.isEmpty() )
    {
        qDebug() << "CompilerThread::testDriver(): m_driverName is empty!";
        return -1;
    }
    
    int ret = QProcess::execute( "modprobe", QStringList() << m_driverName );
    writeLog( "ConnectThread::testDriver(): finished." );
    return ret;
}

void CompilerThread::run()
{
    writeLog( "ConnectThread::run(): called." );

    if ( m_driverName.isEmpty() )
    {
        qDebug() << "CompilerThread::run(): m_driverName is empty!";
        return;
    }

    Compiler::ModemDriver driver;
    QListIterator<Compiler::ModemDriver> it ( m_drivers );

    while ( it.hasNext() )
    {
        driver = it.next();

        if ( driver.compiled_driver_name == ( m_driverName + ".ko" ) )
        {
            Compiler *compiler = new Compiler ( driver, 0 );
            compiler->backupDriver();
            compiler->extractDriver();
            compiler->patchDriver();

            writeLog( "ConnectThread::run(): compiling driver." );  
          
            if ( compiler->compileDriver() == 0 )
            {
                writeLog( "ConnectThread::run(): installing driver." ); 
                
                if ( compiler->installDriver() == 0 )
                {
                    m_result = Success;
                    testDriver();
                }
                else {
                    writeLog( "ConnectThread::run(): driver installation failed!" ); 
                    m_result = InstallationFailed;
                    compiler->restoreDriver();
                }
            }
            else
            {
                compiler->restoreDriver();
                writeLog( "ConnectThread::run(): driver compilation failed!" ); 
                m_result = CompilationFailed;
            }
            
            delete compiler;
            break;
        }
    }

    writeLog( "ConnectThread::run(): finished." ); 
}

void CompilerThread::initDriversMap()
{
    Compiler::ModemDriver sagem_e4;
    sagem_e4.name = "ueagle-atm";
    sagem_e4.version = "1.4";
    sagem_e4.extracted_driver_path = "/tmp/ueagle-atm-1.4";
    sagem_e4.compiled_driver_name = "ueagle-atm.ko";
    sagem_e4.extraction_path = "/tmp";
    sagem_e4.driver_bundle_path = "/opt/ubudsl/drivers/ueagle-atm-1.4.tar";
    sagem_e4.compilation_arguments = "all";
    sagem_e4.installation_arguments = "install";
    sagem_e4.environment_variables = "PWD=/tmp/ueagle-atm-1.4, LC_ALL=C";
    sagem_e4.amd64_compatible = true;

    Compiler::ModemDriver zxdsl;
    zxdsl.name = "unicorn";
    zxdsl.version = "neo";
    zxdsl.extracted_driver_path = "/tmp/unicorn-neo";
    zxdsl.compiled_driver_name = "unicorn_usb_eth.ko";
    zxdsl.extraction_path = "/tmp";
    zxdsl.driver_bundle_path = "/opt/ubudsl/drivers/unicorn-neo.tar";
    zxdsl.compilation_arguments = "clean, all";
    zxdsl.installation_arguments = "install";
    zxdsl.environment_variables = "PWD=/tmp/unicorn-neo, LC_ALL=C";
    zxdsl.amd64_compatible = false;

    m_drivers.append ( sagem_e4 );
    m_drivers.append ( zxdsl );
}

int CompilerThread::getResult()
{
    return m_result;
}
