#include "ConnectThread.h"

//extern bool  debug;
//extern FILE *pFile;


ConnectThread::ConnectThread(QSettings *settings, int (*fGetModemState)(), void (*fWriteLog)(const QString &))
: pppd(new MyProcess(0)),
  getModemState(fGetModemState),
  writeLog(fWriteLog),
  m_env(QProcess::systemEnvironment()),
  m_settings(settings)
{
    writeLog( "ConnectThread::ConnectThread(): called." );

    bool ldLibMet = false;
    bool pathMet  = false;
    
    for ( int i = 0; i < m_env.size(); i++ )
    {
        if ( m_env[i].startsWith( "LD_LIBRARY_PATH=" ) )
        {
            ldLibMet = true;
            m_env[i] = m_env[i].append( ":/opt/ubudsl/lib" );
        }
        else if ( m_env[i].startsWith( "PATH=" ) )
        {
            pathMet = true;
            m_env[i] = m_env[i].append( ":/opt/ubudsl/bin" );
        }
    }
    
    if ( !ldLibMet )
        m_env.append( "/opt/ubudsl/lib" );
    if ( !pathMet )
        m_env.append( "/opt/ubudsl/bin" );
    
    pppd->setEnvironment(m_env);
    pppd->moveToThread(this);
    connect(pppd, SIGNAL(lineAvailable(QByteArray)), this, SIGNAL(debugMsg(QByteArray)));

    writeLog( "ConnectThread::ConnectThread(): finished." );
}

ConnectThread::~ConnectThread()
{
    writeLog( "ConnectThread::~ConnectThread(): called and finished." );
}

void ConnectThread::run()
{
    writeLog( "ConnectThread::run(): called." );
    m_returnState = ReturnState::Unknown;
    
    if ( !waitForSynchronisation() )
    {
        writeLog( "ConnectThread::run(): modem synchronisation failed, returning!" );
        m_returnState = ReturnState::SynchronisationFailed;
        return;
    }
    
    m_settings->sync();
    
    m_arguments.clear();
    m_arguments << "debug"
                << "linkname"
                << "ubudsl"
                << "call";

    if ( m_settings->value("vendor_id", 0).toInt() == 0x0483 )
    {
        m_arguments << "ubudsl_unicorn";
        prepareDSL0();
    }
    else if ( m_settings->value("protocol", "PPPoA").toString() == "PPPoE" )
    {
        m_arguments << "ubudsl";
        prepareBridging();
        prepareNAS0();
    }
    else
    {
        m_arguments << "ubudsl";
    }

    if ( !m_settings->value("pppd_arguments", QStringList()).toStringList().isEmpty() )
        m_arguments += m_settings->value("pppd_arguments").toStringList();
    

    pppd->start("pppd", m_arguments);
    pppd->waitForFinished(90000);

    writeLog( "ConnectThread::run(): finished." );
}

void ConnectThread::prepareNAS0()
{
    writeLog( "ConnectThread::prepareNAS0(): called." );

    QNetworkInterface nas = QNetworkInterface::interfaceFromName("nas0");
    if ( nas.isValid() )
    {
        if ( nas.flags().testFlag(QNetworkInterface::IsUp) )
        {
            writeLog("ConnectThread::prepareNAS0(): interface nas0 is up.");
        }
        else
        {
            writeLog("ConnectThread::prepareNAS0(): interface nas0 is down.");
            QProcess ifconfig;
            ifconfig.start("ifconfig", QStringList() << "nas0" << "up");
            ifconfig.waitForFinished(30000);
            
            if ( ifconfig.exitCode() == 0 )
                writeLog("ConnectThread::prepareNAS0(): ifconfig nas0 up: success.");
            else
                writeLog("ConnectThread::prepareNAS0(): ifconfig nas0 up: failed!");
        }
    }

    writeLog( "ConnectThread::prepareNAS0(): finished." );
}

void ConnectThread::prepareDSL0()
{
    writeLog( "ConnectThread::prepareDSL0(): called." );

    QNetworkInterface dsl = QNetworkInterface::interfaceFromName("dsl0");
    
    if ( dsl.isValid() )
    {
        if ( dsl.flags().testFlag(QNetworkInterface::IsUp) )
        {
            writeLog("ConnectThread::prepareDSL0(): interface dsl0 is up.");
        }
        else
        {
            writeLog("ConnectThread::prepareDSL0(): interface dsl0 is down.");
            QProcess ifconfig;
            ifconfig.moveToThread(this);
            ifconfig.start("ifconfig", QStringList() << "dsl0" << "up");
            ifconfig.waitForFinished(30000);
            
            if ( ifconfig.exitCode() == 0 )
                writeLog("ConnectThread::prepareDSL0(): ifconfig dsl0 up: success.");
            else
                writeLog("ConnectThread::prepareDSL0(): ifconfig dsl0 up: failed!");
        }
    }

    writeLog( "ConnectThread::prepareDSL0(): finished." );
}

void ConnectThread::prepareBridging()
{
    writeLog( "ConnectThread::prepareBridging(): called." );

    QProcess proc;
    proc.moveToThread(this);
    proc.setEnvironment(m_env);
    proc.start("modprobe", QStringList() << "br2684");
    proc.waitForFinished(5000);
    
    const QString VpiVci = m_settings->value("vpi", 0).toString() + "." + m_settings->value("vci", 0).toString();
    writeLog("ConnectThread::prepareBridging(): VPI.VCI: " + VpiVci);

    proc.start( "/opt/ubudsl/bin/br2684ctl", QStringList() << "-b" << "-c" << "0" << "-a" << VpiVci);
    proc.waitForFinished(5000);
    qDebug() << proc.pid();
    
    if ( proc.exitCode() == 0 )
        writeLog("ConnectThread::prepareBridging(): br2684ctl: success.");
    else
        writeLog("ConnectThread::prepareBridging(): br2684ctl: failed!");
    
    QTimer::singleShot(1500, this, SLOT(quit()));
    exec();
    
    //proc.start("ip", QStringList() << "addr" << "add" << "192.168.0.1/32" << "dev" << "nas0" );
    proc.start("ifconfig", QStringList() << "nas0" << "192.168.2.1" <<  "netmask" << "255.255.255.0" );
    proc.waitForFinished(5000);
    
    if ( proc.exitCode() == 0 )
        writeLog("ConnectThread::prepareBridging(): ifconfig: success.");
    else
        writeLog("ConnectThread::prepareBridging(): ifconfig: failed!");

    QTimer::singleShot(1500, this, SLOT(quit()));
    exec();

    writeLog( "ConnectThread::prepareBridging(): finished." );
}

bool ConnectThread::waitForSynchronisation()
{
    static int counter = 0;
    
    if ( getModemState() == Synchronised )
    {
        writeLog("ConnectThread::waitForSynchronisation(): modem synchronised.");
        
        counter = 0;
        return true;
    }
    
    if ( counter > 6 )
    {
        writeLog("ConnectThread::waitForSynchronisation(): counter > 6, resetting and returning.");
        
        counter = 0;
        m_returnState = ReturnState::SynchronisationFailed;
        
        return false;
    }
    
    if ( counter == 0 )
    {
        writeLog("ConnectThread::waitForSynchronisation(): counter == 0, emitting the synchronising signal.");
        
        emit synchronising();
    }
    
    QTimer::singleShot( 5000, this, SLOT(quit()) );
    exec();
    
    counter++;
    
    return waitForSynchronisation();
}

ReturnState::ReturnState ConnectThread::getReturnState()
{
    return m_returnState;
}

void ConnectThread::setModemState(int state)
{
    m_modemState = state;
}
